#!/usr/local/bin/perl -w
############################################################
### Script Name: empl_info.pl
###      Author: Darryl Priest
###        Date: October 2002
### Description: List Info From PMData About Employees
############################################################
$| =1;
use DBI;
use Term::Size;
use Getopt::Std;
use strict;

use lib q{/custom/perl/modules/bin};
use Defaults;

my $Usage = qq#

Usage: empl_info.pl 
	              -b Show Data columns That Are Unpopulated, Default Is To Skip

	              -c Column Name Match To Be Reported

	              -d Database Server To Select PMData Data From

	              -e Employee ID To Report

		      -h Help Information

	              -l Employee Logon ID To Report

	              -n Employee Name To Report

		      -t Include Temporary Employees

		      -v Verbose Column Output

		      -w Lookup Windoze (Active Directory) Data Also

		      -x Include Ex (Terminated) Employees
#;

my ( %opts );
getopts('bc:d:e:ghl:n:tvwx', \%opts);

############################################################
### Just In Case See If There's A Single Argument
############################################################
my @Args = @ARGV;

############################################################
### User Seeks Help, Show It And Exit
############################################################
if ( defined $opts{h} ) {
   show_help();
   exit;
}

############################################################
### Get User Input, Make Sure To Get An Emplid, Name Or Logon
############################################################
my (%In, $include_terms, $show_blanks, $verbose);
my ($active_directory, $include_temps);

if ( defined $opts{b} ) { $show_blanks = 1; }
 else { $show_blanks = 0; }

############################################################
### If Specific columns Are Requested, Make Sure Verbose Is On
### So All Possible columns Are Available, Also Make Sure If 
### Phone Number Is Selected To Include Dial Prefix
############################################################
if ( defined $opts{c} ) {
   $In{columns} = $opts{c};
   if ( $In{columns} =~ /phon/ ) { $In{columns} .= "|dial_prefix"; }
   $opts{v} = 1;
}

if ( defined $opts{d} ) { $In{db} = "pmdata\@$opts{d}"; }
else { $In{db} = "pmdata"; }

if ( defined $opts{e} ) { $In{emplid} = $opts{e}; }

if ( defined $opts{l} ) { $In{logon_id} = lc($opts{l}); }

if ( defined $opts{n} ) { $In{name} = lc($opts{n}); }

if ( defined $opts{t} ) { $include_temps = 1; }
else { $include_temps = 0; }

if ( defined $opts{v} ) { $verbose = 1; }
else { $verbose = 0; }

if ( defined $opts{x} ) { $include_terms = 1; }
else { $include_terms = 0; }

############################################################
### If No Options Were Passed, Check For Valid Argument, 
### Or Die With Usage Displayed
############################################################
if ( ! exists $In{emplid} and ! exists $In{logon_id}  and ! exists $In{name} ) {
   #########################################################
   ### Check The Possible Argument For Possible Usage
   #########################################################
   if ( defined $Args[0] and length($Args[0]) > 1 ) {
      if ( $Args[0] =~ /^[0-9]{5}$/ ) { $In{emplid} = $Args[0]; }
      elsif ( $Args[0] =~ /^[A-Z,a-z,\-\'\"]+$/ ) { $In{name} = lc($Args[0]); }
      elsif ( $Args[0] =~ /^[A-Z,a-z,0-9]{1,8}$/ ) { $In{logon_id} = lc($Args[0]); }
      }
   else {
      die "\n$Usage\n\n";
   }
}

############################################################
### If Seeking Active Directory Information, Connect Now
############################################################
my ($ldap, $ad_mesg);
$active_directory = 0;
if ( defined $opts{w} ) {
   $active_directory = 1;
   use Net::LDAP;
   $ldap = Net::LDAP->new( 'mtwacdc01' ) or die "$@";
   $ad_mesg = $ldap->bind('user@domain.local', password=>'password') or die "$@"
}

############################################################
### If Looking For A Name Make Sure It Has Wild Cards
############################################################
if ( defined $In{name} ) {
   if ( $In{name} !~ /[\*\[\?]/ ) { $In{name} = "*" . $In{name} . "*"; }
}

############################################################
## Get Terminal Width
############################################################
my ($columns, $rows) = Term::Size::chars *STDOUT{IO};
my $PrintWidth = $columns - 2;

############################################################
### Set Default columns String, Which Will Be Reported Unless
### Overridden With -c Or -v
############################################################
my %Defaultcolumns = (
		       assignments        => '',
		       dial_prefix        => '',
		       empl_status_desc   => '',
		       full_prt_time_desc => '',
		       hire_date          => '',
		       job_family         => '',
		       logon_id           => '',
		       published_phones   => '',
		       secretaries        => '',
		       term_date          => '',
		       work_phone         => '',
		     );


############################################################
### Connect To PMData
############################################################
my ($pm_dbh);
if ( defined $opts{d} ) { $pm_dbh = default_db_connect("pmdata", $opts{d}); }
 else { $pm_dbh = default_db_connect("pmdata"); }

############################################################
### Select Emplid & Name For Passed Emplid/Logon/Name Match
############################################################
my ($sql, $where, $term_sql, $temp_sql, $get_emplid_sth, $search_var);
$sql = qq#select emplid, name_first2last, 'E' from pm_empl_search#;

$temp_sql = qq#select emplid, name_first2last, 'T' from pmhr_temps#;
$term_sql = qq#select emplid, name_first2last, 'X' from pmhr_terminations#;

my @sql_parms = ( );

if ( exists $In{emplid} ) {
   $search_var = 'emplid';
   $where = qq# emplid matches ?#;
   }
elsif ( exists $In{logon_id} ) {
   $search_var = 'logon_id';
   $where = qq# lower(logon_id) matches ?#;
   }
elsif ( exists $In{name} ) {
   $where = qq# lower(name_first2last) matches ?#;
   $search_var = 'name';
}

push @sql_parms, $In{ $search_var };
$sql .= qq# where $where#;

if ( $include_temps ) {
   $sql .= qq# union $temp_sql where $where#;
   push @sql_parms, $In{ $search_var };
}

if ( $include_terms ) {
   $sql .= qq# union $term_sql where $where#;
   push @sql_parms, $In{ $search_var };
}
$sql .= qq# order by 2,1#;

$get_emplid_sth = $pm_dbh->prepare($sql);
$get_emplid_sth->execute(@sql_parms);

############################################################
### Fetch All Employees Found For Passed Match
############################################################
my $empl_ref = $get_emplid_sth->fetchall_arrayref();

############################################################
### If Only Employee Matches, Call Show Subroutine, Else
### Show List Of Matching Employees And Allow User To 
### Select In A Loop From The List And Report
############################################################
my $lists_shown = 0;
if ( @{$empl_ref} > 0 ) {
   if ( @{$empl_ref} == 1 ) {
      list_empl_info($empl_ref->[0][0], $empl_ref->[0][2]);
      }
    else {
      show_list($empl_ref);
      my ($Choice);
      while (<STDIN>) {
         chomp;

         if ( $_ =~ /[Xx]/ ) { last; }

         $Choice = $_ - 1;
	 if ( $Choice < @{$empl_ref} ) {
            list_empl_info($empl_ref->[${Choice}][0], $empl_ref->[${Choice}][2]);
	 }
         show_list($empl_ref);
      }
   } }
 else {
   print "\n\nNo Matches Found For Passed Criteria\n\n";
}

$pm_dbh->disconnect();

if ( $active_directory ) { $ad_mesg = $ldap->unbind; }
############################################################
###                End Of Main Program                   ###
############################################################


############################################################
### SubRoutine: show_list
### This subroutine will list the passed list reference
### of employee ids and names.
 ############################################################
sub show_list {
my ($ListRef) = @_;

############################################################
### If This Isn't The First Time This Was Called
############################################################
if ( $lists_shown > 0 ) {
   print "Press Enter To Continue";
   while (<STDIN>) { last; }
}
$lists_shown++;

my ($x, $y);
print "\n\n      Selected Employees\n";
print "      -------- ---------\n";
for ($x = 0; $x < @{$ListRef}; $x++) {
   $y = $x + 1;
   if ( $ListRef->[$x][2] eq "E" ) {
      printf("%3d.) %s (%s)\n",  $y, $ListRef->[$x][1], $ListRef->[$x][0]); }
    elsif ( $ListRef->[$x][2] eq "T" ) {
      printf("%3d.) %s (%s) - Temp/Contractor\n",  $y, $ListRef->[$x][1], $ListRef->[$x][0]); }
    else {
      printf("%3d.) %s (%s) - Terminated\n",  $y, $ListRef->[$x][1], $ListRef->[$x][0]); 
   }
}
print "\nEnter Choice(or x to exit): ";
} ### End Of SubRoutine show_list


############################################################
### SubRoutine: list_empl_info
### This subroutine will list the employee information
### from pm_employees_v or pmhr_terminations based on
### employee status for the passed emplid.
############################################################
sub list_empl_info {

my ($this_emplid, $empl_status) = @_;

############################################################
### Select All Potential Data columns For Passed Emplid
############################################################
if ( $empl_status eq "E" ) { 
   $sql = qq#select * from pm_employees_v where emplid = ?#;
   }
 elsif ( $empl_status eq "T" ) { 
   $sql = qq#select emplid, hire_date, name_last2first, name_first2last, preferred_name,
		    logon_id, job_family, jobcode, long_jobtitle, short_jobtitle,
		    location, location_desc, acct_location, deptid, deptname,
		    practice_group, dial_prefix, work_phone, cell_phone,
		    supervisor_id, supervisor_name,
		    assignment_ids, assignment_names, assignments,
		    secretary_id, secretary_name,
		    secretary_ids, secretary_names, secretaries
	     from pmhr_temps
	     where emplid = ?#;
   }
 else {
   $sql = qq#select emplid, term_date, name_last2first, name_first2last,
                    logon_id, sex_desc, ethnic_group_desc, job_family,
                    jobcode, long_jobtitle, short_jobtitle,
                    location, location_desc, acct_location,
                    deptid, deptname, practice_group
	     from pmhr_terminations
	     where emplid = ?#;
}

my $get_pmdata_sth = $pm_dbh->prepare_cached($sql);

$get_pmdata_sth->execute($this_emplid);

############################################################
### Define Output format For Employee Data, The format Is
### Defined Dymanically So It Will Fit The Screen Width
### (Each Output Line Of The format Is On One Line Below)
### Only Define The Funtion The First Time Through Though
############################################################
my ($format, $empl_row, $var, $out_var, $out_value);

if ( $lists_shown <= 1 ) {
$format = "format STDOUT = \n" .
          " @>>>>>>>>>>>>>>>>>: ^" . "<" x ($PrintWidth - 21) . "\n" .
          '$out_var' . ",               " . '$out_value' . "\n" .
          " ~~                  ^" . "<" x ($PrintWidth - 21) . "\n" .
          "                    " . '$out_value' . "\n" .
          ".\n";

eval $format;
}

while ( $empl_row = $get_pmdata_sth->fetchrow_hashref() ) {
   #########################################################
   ### Print "Header" Of Employee Information
   #########################################################
   print ">" x $PrintWidth, "\n";
   for $var ( qw(emplid name_first2last location_desc long_jobtitle) ) {
      $out_var = $var;
      $out_value = $empl_row->{$var};
      write;
#      printf(" %18s: %s\n", $var, $empl_row->{$var});
   }
   print "\n";

   #########################################################
   ### For Each Returned Column
   #########################################################
   for $var ( sort keys %{$empl_row} ) {
      if ( $var =~ /_s$/ ) { next; }

      ######################################################
      ### If User Selected Specific columns To Report, Only
      ### Report The Selected columns
      ######################################################
      if ( exists $In{columns} ) {
	 if ( $var !~ /$In{columns}/ ) { next; }
      }

      ######################################################
      ### If Not Verbose And This Column Isn't A Default, Skip It
      ######################################################
      if ( ! $verbose ) {
	 if ( ! exists $Defaultcolumns{ $var } ) { next; }
      }

      ######################################################
      ### If Column Contains Data, Report It, Unless Blanks
      ### Are To Be Shown, Then Set It To "" & Print It Anyway
      ######################################################
      if ( $show_blanks ) { 
	 if ( ! defined $empl_row->{$var} ) { $empl_row->{$var} = ""; }
      }

      if ( $show_blanks or 
	 ( defined $empl_row->{$var} and length($empl_row->{$var}) > 0 ) ) {
	 $out_var = $var;
	 $out_value = $empl_row->{$var};
         write;
      }
   }

   #########################################################
   ### If Active Directory Data Is To Be Shown, Try To Find
   ### It, First By LPR Logon From PMData
   #########################################################
   if ( $active_directory ) {
      print "\n";
      $out_var = q{};
      $out_value = "Active Directory Information";
      write;
      my ($ad_count, $ad_entry, @ad_values);
      my @ad_keys = qw( displayname
			employeeid
			mail
			memberof
			proxyaddresses
			userprincipalname
		       );

      $ad_mesg = $ldap->search(
	 base   => "dc=dc,dc=root,dc=local",
         filter => "(&(userprincipalname=$empl_row->{logon_id}*) (objectClass=User))",
         attrs  => [ @ad_keys ]);

      $ad_count = $ad_mesg->count;

      ######################################################
      ### If An Entry Was Found In AD, Report It
      ######################################################
      if ( $ad_count == 1 ) {
	 $ad_entry = $ad_mesg->shift_entry;

	 foreach $out_var ( @ad_keys ) {
	    @ad_values = $ad_entry->get_value($out_var);
	    if ( @ad_values == 0 ) { $ad_values[0] = 'Undefined In AD'; }
            foreach $out_value ( @ad_values ) {
	       $out_value =~ s/,DC=dc,DC=Root,DC=Local//;
	       write;
	    }
	 }
      }
   }
   print "<" x $PrintWidth, "\n";
}
} ### End Of SubRoutine list_empl_info


############################################################
############################################################
sub show_help {

print $Usage;

print qq#
   -b  This option will report data columns from PMData, even if they are
       empty. The default behaviour is to not print empty/null values.

   -c  Using this option will allow for specific columns to be selected and
       reported from PMData. By default only a few key data columns are
       reported. If -c is used, the key identification columns are reported
       followed by columns with match the patten passed as the option to -c.

   -d  Using this options allow selection from a PMData database on a remote
       server. This is particularly useful when running on a test database
       and wishing to query a current PMData.

   -e  The employee id(s) to report, pattern matches are accepted, provided
       they are can be parsed by the sql engine.

   -h  Diplay this help information.

   -l  The logon id(s) of the employees who are to be reported, pattern matches
       are accepted, provided they are can be parsed by the sql engine. Logon
       id based searches are not case sensitive.

   -n  The name of the employees who are to be reported, pattern matches are
       accepted, provided they are can be parsed by the sql engine. Name
       based searches are case-insensitive.

   -t  This option will include temporary employees in the search. This will
       search the pmhr_temps table with the -e, -l or -n options.

   -v  This option will report "all" columns from the table being reported, 
       either pmhr_employees, pmhr_temps or pmhr_terminations.

   -w  The -w option will attempt to report Active Directory information for
       the person being reported. It will first search AD using the "current"
       logon_id.
       The "columns" reported from AD are displayname, employeeid, mail,
       memberof, proxyaddresses and userprincipalname.

   -x  This option will include terminated employees in the search. This will
       search the pmhr_terminations table with the -e, -l or -n options.
#;

} ### End Of SubRoutine show_help
